# This file was auto-generated by Fern from our API Definition.

import typing

PiiCategory = typing.Union[
    typing.Literal[
        "Name",
        "Email",
        "PhoneNumber",
        "StreetAddress",
        "CreditCardNumber",
        "UsBankAccountNumber",
        "UsSocialSecurityNumber",
        "UsUkPassportNumber",
        "UsDriversLicenseNumber",
        "UsIndividualTaxpayerIdentification",
        "Date",
        "IpAddress",
        "Url",
        "AbaRoutingNumber",
        "Age",
        "SwiftCode",
        "AuBankAccountNumber",
        "AuBusinessNumber",
        "AuCompanyNumber",
        "AuDriversLicenseNumber",
        "AuMedicalAccountNumber",
        "AuPassportNumber",
        "AuTaxFileNumber",
        "CaBankAccountNumber",
        "CaDriversLicenseNumber",
        "CaHealthServiceNumber",
        "CaPassportNumber",
        "CaPersonalHealthIdentification",
        "CaSocialInsuranceNumber",
        "EsDni",
        "EsSocialSecurityNumber",
        "EsTaxIdentificationNumber",
        "EuDebitCardNumber",
        "EuDriversLicenseNumber",
        "EuGpsCoordinates",
        "EuNationalIdentificationNumber",
        "EuPassportNumber",
        "EuSocialSecurityNumber",
        "EuTaxIdentificationNumber",
        "FrDriversLicenseNumber",
        "FrHealthInsuranceNumber",
        "FrNationalId",
        "FrPassportNumber",
        "FrSocialSecurityNumber",
        "FrTaxIdentificationNumber",
        "FrValueAddedTaxNumber",
        "InternationalBankingAccountNumber",
        "NzBankAccountNumber",
        "NzDriversLicenseNumber",
        "NzInlandRevenueNumber",
        "NzMinistryOfHealthNumber",
        "NzSocialWelfareNumber",
        "UkDriversLicenseNumber",
        "UkElectoralRollNumber",
        "UkNationalHealthNumber",
        "UkNationalInsuranceNumber",
        "UkUniqueTaxpayerNumber",
    ],
    typing.Any,
]
