# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import TranslationRequest, TranslationResponse

__all__ = ["TranslationRequest", "TranslationResponse"]
