# This file was auto-generated by Fern from our API Definition.

import typing

UserEventName = typing.Union[
    typing.Literal["BUTTON_CLICKED", "LINK_CLICKED", "CHAT_OPENED", "CHAT_CLOSED", "TEXT_INSERTED"], typing.Any
]
