# This file was auto-generated by Fern from our API Definition.

import typing

from .action_form_attachment import ActionFormAttachment

ActionFormRequestParamValue = typing.Union[typing.Optional[typing.Any], ActionFormAttachment]
