# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ConversationMetadata(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.conversation import ConversationMetadata

    ConversationMetadata(
        metadata={
            "myapp": {"key": "newValue"},
            "conversation-owning-app": {"existingKey": "existingValue"},
        },
    )
    """

    metadata: typing.Dict[str, typing.Dict[str, str]] = pydantic.Field()
    """
    All metadata for the conversation. Keyed by appId.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
