# This file was auto-generated by Fern from our API Definition.

import typing

MessageStatus = typing.Union[typing.Literal["SENDING", "SENT", "REJECTED", "CANCELED", "FAILED", "UNKNOWN"], typing.Any]
