# This file was auto-generated by Fern from our API Definition.

import typing

InboxItemStatus = typing.Union[
    typing.Literal["OPEN", "USER_RESOLVED", "SYSTEM_RESOLVED", "REGRESSED", "IGNORED"], typing.Any
]
