# This file was auto-generated by Fern from our API Definition.

import typing

AgentCapabilityField = typing.Union[
    typing.Literal["CreatedAt", "Enabled", "Name", "Type", "UserInteractionRequired"], typing.Any
]
