# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .entity_id_without_agent import EntityIdWithoutAgent


class BotLogicKnowledgeDetail(UniversalBaseModel):
    knowledge_base_id: typing_extensions.Annotated[EntityIdWithoutAgent, FieldMetadata(alias="knowledgeBaseId")]
    document_id: typing_extensions.Annotated[EntityIdWithoutAgent, FieldMetadata(alias="documentId")]
    document_name: typing_extensions.Annotated[str, FieldMetadata(alias="documentName")]
    document_excerpt: typing_extensions.Annotated[str, FieldMetadata(alias="documentExcerpt")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
