# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    SegmentBase,
    SegmentField,
    SegmentPatchRequest,
    SegmentRequest,
    SegmentResponse,
    SegmentStatus,
    SegmentsSearchRequest,
    SegmentsSearchResponse,
)

__all__ = [
    "SegmentBase",
    "SegmentField",
    "SegmentPatchRequest",
    "SegmentRequest",
    "SegmentResponse",
    "SegmentStatus",
    "SegmentsSearchRequest",
    "SegmentsSearchResponse",
]
