# This file was auto-generated by Fern from our API Definition.

import typing

LlmPersona = typing.Union[
    typing.Literal[
        "CASUAL_BUDDY",
        "EMPATHETIC_SUPPORTER",
        "FORMAL_PROFESSIONAL",
        "CONCISE_EXPERT",
        "ENTHUSIASTIC_HELPER",
        "PATIENT_EDUCATOR",
        "PIRATE",
    ],
    typing.Any,
]
