# This file was auto-generated by Fern from our API Definition.

import typing

Capability = typing.Union[
    typing.Literal["MARKDOWN", "FORMS", "IMAGES", "CHARTS_HIGHCHARTS_TS", "ASYNC", "OAUTH_BUTTONS"], typing.Any
]
