# This file was auto-generated by Fern from our API Definition.

import typing

TriggerType = typing.Union[
    typing.Literal["CONVERSATION_CREATED", "FEEDBACK_CREATED", "INBOX_ITEM_CREATED", "EVENT_CREATED"], typing.Any
]
