# This file was auto-generated by Fern from our API Definition.

import typing

ActionField = typing.Union[
    typing.Literal["AppId", "Name", "LlmInclusionStatus", "UserInteractionRequired", "CreatedAt"], typing.Any
]
