# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from .conversation_information import ConversationInformation
from .inbox_item_base import InboxItemBase
from .inbox_item_fix_add_document import InboxItemFixAddDocument


class InboxItemMissingKnowledge(InboxItemBase):
    fix: InboxItemFixAddDocument = pydantic.Field()
    """
    Fix associated with the inbox item.
    """

    conversations: typing.List[ConversationInformation] = pydantic.Field()
    """
    List of Conversation information objects related to the inbox item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
